@@ -102,6 +102,15 @@ QString serviceName()
 }
 }
 
+void KGlobalAccelPrivate::cleanup()
+{
+    qDeleteAll(components);
+    delete m_iface;
+    m_iface = nullptr;
+    delete m_watcher;
+    m_watcher = nullptr;
+}
+
 KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *q)
     :
 #ifndef KGLOBALACCEL_NO_DEPRECATED
@@ -110,11 +119,11 @@ KGlobalAccelPrivate::KGlobalAccelPrivate(KGlobalAccel *q)
     q(q),
     m_iface(Q_NULLPTR)
 {
-    QDBusServiceWatcher *watcher = new QDBusServiceWatcher(serviceName(),
+    m_watcher = new QDBusServiceWatcher(serviceName(),
             QDBusConnection::sessionBus(),
             QDBusServiceWatcher::WatchForOwnerChange,
             q);
-    q->connect(watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
+    q->connect(m_watcher, SIGNAL(serviceOwnerChanged(QString,QString,QString)),
                q, SLOT(_k_serviceOwnerChanged(QString,QString,QString)));
 }
 
@@ -124,7 +133,7 @@ org::kde::KGlobalAccel *KGlobalAccelPrivate::iface()
         m_iface = new org::kde::KGlobalAccel(serviceName(), QStringLiteral("/kglobalaccel"), QDBusConnection::sessionBus());
         // Make sure kglobalaccel is running. The iface declaration above somehow works anyway.
         QDBusConnectionInterface *bus = QDBusConnection::sessionBus().interface();
-        if (!bus->isServiceRegistered(serviceName())) {
+        if (bus && !bus->isServiceRegistered(serviceName())) {
             QDBusReply<void> reply = bus->startService(serviceName());
             if (!reply.isValid()) {
                 qCritical() << "Couldn't start kglobalaccel from org.kde.kglobalaccel.service:" << reply.error();
@@ -204,11 +213,20 @@ void KGlobalAccel::setEnabled(bool enabled)
 class KGlobalAccelSingleton
 {
 public:
+    KGlobalAccelSingleton();
+
     KGlobalAccel instance;
 };
 
 Q_GLOBAL_STATIC(KGlobalAccelSingleton, s_instance)
 
+KGlobalAccelSingleton::KGlobalAccelSingleton()
+{
+    qAddPostRoutine([]() {
+        s_instance->instance.d->cleanup();
+    });
+}
+
 KGlobalAccel *KGlobalAccel::self()
 {
     return &s_instance()->instance;
